# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .task import (
    TaskResource,
    AsyncTaskResource,
    TaskResourceWithRawResponse,
    AsyncTaskResourceWithRawResponse,
    TaskResourceWithStreamingResponse,
    AsyncTaskResourceWithStreamingResponse,
)
from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .profiles import (
    ProfilesResource,
    AsyncProfilesResource,
    ProfilesResourceWithRawResponse,
    AsyncProfilesResourceWithRawResponse,
    ProfilesResourceWithStreamingResponse,
    AsyncProfilesResourceWithStreamingResponse,
)
from .sessions import (
    SessionsResource,
    AsyncSessionsResource,
    SessionsResourceWithRawResponse,
    AsyncSessionsResourceWithRawResponse,
    SessionsResourceWithStreamingResponse,
    AsyncSessionsResourceWithStreamingResponse,
)
from .extensions import (
    ExtensionsResource,
    AsyncExtensionsResource,
    ExtensionsResourceWithRawResponse,
    AsyncExtensionsResourceWithRawResponse,
    ExtensionsResourceWithStreamingResponse,
    AsyncExtensionsResourceWithStreamingResponse,
)
from .identities import (
    IdentitiesResource,
    AsyncIdentitiesResource,
    IdentitiesResourceWithRawResponse,
    AsyncIdentitiesResourceWithRawResponse,
    IdentitiesResourceWithStreamingResponse,
    AsyncIdentitiesResourceWithStreamingResponse,
)
from .applications import (
    ApplicationsResource,
    AsyncApplicationsResource,
    ApplicationsResourceWithRawResponse,
    AsyncApplicationsResourceWithRawResponse,
    ApplicationsResourceWithStreamingResponse,
    AsyncApplicationsResourceWithStreamingResponse,
)

__all__ = [
    "ProfilesResource",
    "AsyncProfilesResource",
    "ProfilesResourceWithRawResponse",
    "AsyncProfilesResourceWithRawResponse",
    "ProfilesResourceWithStreamingResponse",
    "AsyncProfilesResourceWithStreamingResponse",
    "SessionsResource",
    "AsyncSessionsResource",
    "SessionsResourceWithRawResponse",
    "AsyncSessionsResourceWithRawResponse",
    "SessionsResourceWithStreamingResponse",
    "AsyncSessionsResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "ExtensionsResource",
    "AsyncExtensionsResource",
    "ExtensionsResourceWithRawResponse",
    "AsyncExtensionsResourceWithRawResponse",
    "ExtensionsResourceWithStreamingResponse",
    "AsyncExtensionsResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "TaskResource",
    "AsyncTaskResource",
    "TaskResourceWithRawResponse",
    "AsyncTaskResourceWithRawResponse",
    "TaskResourceWithStreamingResponse",
    "AsyncTaskResourceWithStreamingResponse",
    "IdentitiesResource",
    "AsyncIdentitiesResource",
    "IdentitiesResourceWithRawResponse",
    "AsyncIdentitiesResourceWithRawResponse",
    "IdentitiesResourceWithStreamingResponse",
    "AsyncIdentitiesResourceWithStreamingResponse",
    "ApplicationsResource",
    "AsyncApplicationsResource",
    "ApplicationsResourceWithRawResponse",
    "AsyncApplicationsResourceWithRawResponse",
    "ApplicationsResourceWithStreamingResponse",
    "AsyncApplicationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, TypeAlias, TypedDict

__all__ = [
    "IdentityUpdateParams",
    "Credential",
    "CredentialUsernamePasswordCredentialSchema",
    "CredentialAuthenticatorCredentialSchema",
    "CredentialCustomCredentialSchema",
    "CredentialCustomCredentialSchemaField",
]


class IdentityUpdateParams(TypedDict, total=False):
    credentials: Iterable[Credential]
    """Array of credentials for authentication"""

    metadata: Dict[str, object]
    """Metadata for the identity"""

    name: str
    """Name of the identity"""


class CredentialUsernamePasswordCredentialSchema(TypedDict, total=False):
    password: str
    """The password of the credential"""

    type: Literal["username_password"]
    """The type of credential"""

    username: str
    """The username of the credential"""


class CredentialAuthenticatorCredentialSchema(TypedDict, total=False):
    otp: str
    """The OTP of the credential"""

    secret: str
    """The secret of the credential"""

    type: Literal["authenticator"]
    """The type of credential"""


class CredentialCustomCredentialSchemaField(TypedDict, total=False):
    name: str
    """The name of the field"""

    value: str
    """The value of the field"""


class CredentialCustomCredentialSchema(TypedDict, total=False):
    fields: Iterable[CredentialCustomCredentialSchemaField]

    type: Literal["custom"]
    """The type of credential"""


Credential: TypeAlias = Union[
    CredentialUsernamePasswordCredentialSchema,
    CredentialAuthenticatorCredentialSchema,
    CredentialCustomCredentialSchema,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SessionDragAndDropParams"]


class SessionDragAndDropParams(TypedDict, total=False):
    end_x: Required[Annotated[int, PropertyInfo(alias="endX")]]
    """Ending X coordinate"""

    end_y: Required[Annotated[int, PropertyInfo(alias="endY")]]
    """Ending Y coordinate"""

    start_x: Required[Annotated[int, PropertyInfo(alias="startX")]]
    """Starting X coordinate"""

    start_y: Required[Annotated[int, PropertyInfo(alias="startY")]]
    """Starting Y coordinate"""

    button: Literal["left", "middle", "right"]
    """Mouse button to use"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["IdentityDeleteResponse"]


class IdentityDeleteResponse(BaseModel):
    message: Optional[str] = None
    """Deletion result message"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.applications import auth_flow_create_params, auth_flow_update_params
from ...types.applications.auth_flow_list_response import AuthFlowListResponse
from ...types.applications.auth_flow_create_response import AuthFlowCreateResponse
from ...types.applications.auth_flow_delete_response import AuthFlowDeleteResponse
from ...types.applications.auth_flow_update_response import AuthFlowUpdateResponse

__all__ = ["AuthFlowsResource", "AsyncAuthFlowsResource"]


class AuthFlowsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthFlowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AuthFlowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthFlowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AuthFlowsResourceWithStreamingResponse(self)

    def create(
        self,
        application_id: str,
        *,
        methods: List[Literal["username_password", "authenticator", "custom"]],
        name: str,
        custom_fields: Iterable[auth_flow_create_params.CustomField] | Omit = omit,
        description: str | Omit = omit,
        is_recommended: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthFlowCreateResponse:
        """Creates a new authentication flow for an application.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          methods: Authentication methods in this flow

          name: Name of the authentication flow

          custom_fields: Custom fields for this authentication flow

          description: Description of the authentication flow

          is_recommended: Whether this is the recommended authentication flow

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._post(
            f"/v1/applications/{application_id}/auth-flows",
            body=maybe_transform(
                {
                    "methods": methods,
                    "name": name,
                    "custom_fields": custom_fields,
                    "description": description,
                    "is_recommended": is_recommended,
                },
                auth_flow_create_params.AuthFlowCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthFlowCreateResponse,
        )

    def update(
        self,
        auth_flow_id: str,
        *,
        application_id: str,
        custom_fields: Iterable[auth_flow_update_params.CustomField] | Omit = omit,
        description: str | Omit = omit,
        is_recommended: bool | Omit = omit,
        methods: List[Literal["username_password", "authenticator", "custom"]] | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthFlowUpdateResponse:
        """Updates an existing authentication flow.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          custom_fields: Custom fields for this authentication flow

          description: Description of the authentication flow

          is_recommended: Whether this is the recommended authentication flow

          methods: Authentication methods in this flow

          name: Name of the authentication flow

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        if not auth_flow_id:
            raise ValueError(f"Expected a non-empty value for `auth_flow_id` but received {auth_flow_id!r}")
        return self._patch(
            f"/v1/applications/{application_id}/auth-flows/{auth_flow_id}",
            body=maybe_transform(
                {
                    "custom_fields": custom_fields,
                    "description": description,
                    "is_recommended": is_recommended,
                    "methods": methods,
                    "name": name,
                },
                auth_flow_update_params.AuthFlowUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthFlowUpdateResponse,
        )

    def list(
        self,
        application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthFlowListResponse:
        """
        Retrieves all authentication flows for a specific application.

        **Beta** Capability. [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._get(
            f"/v1/applications/{application_id}/auth-flows",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthFlowListResponse,
        )

    def delete(
        self,
        auth_flow_id: str,
        *,
        application_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthFlowDeleteResponse:
        """Deletes an existing authentication flow.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        if not auth_flow_id:
            raise ValueError(f"Expected a non-empty value for `auth_flow_id` but received {auth_flow_id!r}")
        return self._delete(
            f"/v1/applications/{application_id}/auth-flows/{auth_flow_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthFlowDeleteResponse,
        )


class AsyncAuthFlowsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthFlowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthFlowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthFlowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncAuthFlowsResourceWithStreamingResponse(self)

    async def create(
        self,
        application_id: str,
        *,
        methods: List[Literal["username_password", "authenticator", "custom"]],
        name: str,
        custom_fields: Iterable[auth_flow_create_params.CustomField] | Omit = omit,
        description: str | Omit = omit,
        is_recommended: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthFlowCreateResponse:
        """Creates a new authentication flow for an application.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          methods: Authentication methods in this flow

          name: Name of the authentication flow

          custom_fields: Custom fields for this authentication flow

          description: Description of the authentication flow

          is_recommended: Whether this is the recommended authentication flow

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._post(
            f"/v1/applications/{application_id}/auth-flows",
            body=await async_maybe_transform(
                {
                    "methods": methods,
                    "name": name,
                    "custom_fields": custom_fields,
                    "description": description,
                    "is_recommended": is_recommended,
                },
                auth_flow_create_params.AuthFlowCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthFlowCreateResponse,
        )

    async def update(
        self,
        auth_flow_id: str,
        *,
        application_id: str,
        custom_fields: Iterable[auth_flow_update_params.CustomField] | Omit = omit,
        description: str | Omit = omit,
        is_recommended: bool | Omit = omit,
        methods: List[Literal["username_password", "authenticator", "custom"]] | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthFlowUpdateResponse:
        """Updates an existing authentication flow.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          custom_fields: Custom fields for this authentication flow

          description: Description of the authentication flow

          is_recommended: Whether this is the recommended authentication flow

          methods: Authentication methods in this flow

          name: Name of the authentication flow

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        if not auth_flow_id:
            raise ValueError(f"Expected a non-empty value for `auth_flow_id` but received {auth_flow_id!r}")
        return await self._patch(
            f"/v1/applications/{application_id}/auth-flows/{auth_flow_id}",
            body=await async_maybe_transform(
                {
                    "custom_fields": custom_fields,
                    "description": description,
                    "is_recommended": is_recommended,
                    "methods": methods,
                    "name": name,
                },
                auth_flow_update_params.AuthFlowUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthFlowUpdateResponse,
        )

    async def list(
        self,
        application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthFlowListResponse:
        """
        Retrieves all authentication flows for a specific application.

        **Beta** Capability. [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._get(
            f"/v1/applications/{application_id}/auth-flows",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthFlowListResponse,
        )

    async def delete(
        self,
        auth_flow_id: str,
        *,
        application_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthFlowDeleteResponse:
        """Deletes an existing authentication flow.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        if not auth_flow_id:
            raise ValueError(f"Expected a non-empty value for `auth_flow_id` but received {auth_flow_id!r}")
        return await self._delete(
            f"/v1/applications/{application_id}/auth-flows/{auth_flow_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthFlowDeleteResponse,
        )


class AuthFlowsResourceWithRawResponse:
    def __init__(self, auth_flows: AuthFlowsResource) -> None:
        self._auth_flows = auth_flows

        self.create = to_raw_response_wrapper(
            auth_flows.create,
        )
        self.update = to_raw_response_wrapper(
            auth_flows.update,
        )
        self.list = to_raw_response_wrapper(
            auth_flows.list,
        )
        self.delete = to_raw_response_wrapper(
            auth_flows.delete,
        )


class AsyncAuthFlowsResourceWithRawResponse:
    def __init__(self, auth_flows: AsyncAuthFlowsResource) -> None:
        self._auth_flows = auth_flows

        self.create = async_to_raw_response_wrapper(
            auth_flows.create,
        )
        self.update = async_to_raw_response_wrapper(
            auth_flows.update,
        )
        self.list = async_to_raw_response_wrapper(
            auth_flows.list,
        )
        self.delete = async_to_raw_response_wrapper(
            auth_flows.delete,
        )


class AuthFlowsResourceWithStreamingResponse:
    def __init__(self, auth_flows: AuthFlowsResource) -> None:
        self._auth_flows = auth_flows

        self.create = to_streamed_response_wrapper(
            auth_flows.create,
        )
        self.update = to_streamed_response_wrapper(
            auth_flows.update,
        )
        self.list = to_streamed_response_wrapper(
            auth_flows.list,
        )
        self.delete = to_streamed_response_wrapper(
            auth_flows.delete,
        )


class AsyncAuthFlowsResourceWithStreamingResponse:
    def __init__(self, auth_flows: AsyncAuthFlowsResource) -> None:
        self._auth_flows = auth_flows

        self.create = async_to_streamed_response_wrapper(
            auth_flows.create,
        )
        self.update = async_to_streamed_response_wrapper(
            auth_flows.update,
        )
        self.list = async_to_streamed_response_wrapper(
            auth_flows.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            auth_flows.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal

import httpx

from ..types import tool_fetch_webpage_params, tool_perform_web_task_params, tool_screenshot_webpage_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.tool_perform_web_task_response import ToolPerformWebTaskResponse

__all__ = ["ToolsResource", "AsyncToolsResource"]


class ToolsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return ToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return ToolsResourceWithStreamingResponse(self)

    def fetch_webpage(
        self,
        *,
        session_id: str | Omit = omit,
        format: Literal["html", "markdown"] | Omit = omit,
        new_page: bool | Omit = omit,
        page_index: int | Omit = omit,
        return_partial_on_timeout: bool | Omit = omit,
        url: str | Omit = omit,
        wait: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Retrieve the rendered content of a webpage, optionally formatted as Markdown or
        HTML.

        Args:
          session_id: An optional browser session identifier to reference an existing running browser
              session. If provided, the tool will execute within that browser session.

          format: The output format of the content.

          new_page: Whether to create a new page for the content.

          page_index: The index of the page to fetch content from. **Overides new_page**.

          return_partial_on_timeout: Whether to return partial content if the content is not loaded within the 20
              seconds.

          url: The URL of the webpage to fetch content from. When left empty, the current
              webpage is used.

          wait: The time to wait for **dynamic** content to load in **milliseconds**.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return self._post(
            "/v1/tools/fetch-webpage",
            body=maybe_transform(
                {
                    "format": format,
                    "new_page": new_page,
                    "page_index": page_index,
                    "return_partial_on_timeout": return_partial_on_timeout,
                    "url": url,
                    "wait": wait,
                },
                tool_fetch_webpage_params.ToolFetchWebpageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"session_id": session_id}, tool_fetch_webpage_params.ToolFetchWebpageParams),
            ),
            cast_to=str,
        )

    def perform_web_task(
        self,
        *,
        prompt: str,
        session_id: str | Omit = omit,
        agent: Literal["browser-use", "openai-cua"] | Omit = omit,
        detect_elements: bool | Omit = omit,
        highlight_elements: bool | Omit = omit,
        human_intervention: bool | Omit = omit,
        max_steps: int | Omit = omit,
        model: str | Omit = omit,
        output_schema: object | Omit = omit,
        provider: Literal["openai", "gemini", "groq", "azure", "xai"] | Omit = omit,
        secret_values: Dict[str, str] | Omit = omit,
        url: str | Omit = omit,
        directly_open_url: bool | Omit = omit,
        async_: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolPerformWebTaskResponse:
        """
        Start from a URL and perform the given task.

        Args:
          prompt: The task to be autonomously completed.

          session_id: An optional browser session identifier to reference an existing running browser
              sessions. When passed, the tool will be executed on the provided browser
              session.

          agent: The AI agent to use for task completion. Defaults to browser-use.

          detect_elements: Enable element detection for better interaction accuracy. Improves the agent's
              ability to identify and interact with UI elements.

          highlight_elements: Whether to highlight elements during task execution for better visibility.

          human_intervention: Allow human intervention during task execution. When enabled, the agent can
              request human input for ambiguous situations.

          max_steps: Maximum number of steps the agent can take to complete the task. Defaults
              to 200.

          model: The specific model to use for task completion. see our
              [models](/agentic-browser-control/ai-task-completion#available-models) page for
              more information.

          output_schema: JSON Schema defining the expected structure of the output data.

          provider: The AI provider to use for task completion.

          secret_values: Secret values to pass to the agent for secure credential handling. Keys and
              values are passed as environment variables to the agent.

          url: The URL of the webpage. If not provided, the tool will use the current page in
              the session.

          directly_open_url: If true, the tool will directly open the URL in the browser.

          async_: Whether to execute the task asynchronously.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/tools/perform-web-task",
            body=maybe_transform(
                {
                    "prompt": prompt,
                    "agent": agent,
                    "detect_elements": detect_elements,
                    "highlight_elements": highlight_elements,
                    "human_intervention": human_intervention,
                    "max_steps": max_steps,
                    "model": model,
                    "output_schema": output_schema,
                    "provider": provider,
                    "secret_values": secret_values,
                    "url": url,
                    "directly_open_url": directly_open_url,
                    "async": async_,
                },
                tool_perform_web_task_params.ToolPerformWebTaskParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"session_id": session_id}, tool_perform_web_task_params.ToolPerformWebTaskParams
                ),
            ),
            cast_to=ToolPerformWebTaskResponse,
        )

    def screenshot_webpage(
        self,
        *,
        session_id: str | Omit = omit,
        capture_full_height: bool | Omit = omit,
        height: int | Omit = omit,
        image_quality: int | Omit = omit,
        s3_target_address: str | Omit = omit,
        scroll_all_content: bool | Omit = omit,
        url: str | Omit = omit,
        wait: int | Omit = omit,
        width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        This endpoint captures a screenshot of the specified webpage using Chromium.
        Users can customize the viewport dimensions and capture options.

        Args:
          session_id: An optional browser session identifier to reference an existing running browser
              sessions. When passed, the tool will be executed on the provided browser
              session.

          capture_full_height: If true, captures the entire height of the page, ignoring the viewport height.

          height: The height of the browser viewport in pixels.

          image_quality: Quality of the output image, on the range 1-100. 100 will not perform any
              compression.

          s3_target_address: Presigned S3 url target to upload the image to.

          scroll_all_content: If true, scrolls the page and captures all visible content.

          url: The URL of the webpage to capture.

          wait: Duration in milliseconds to wait after page has loaded, mainly used for sites
              with JS animations.

          width: The width of the browser viewport in pixels.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "image/png", **(extra_headers or {})}
        return self._post(
            "/v1/tools/screenshot",
            body=maybe_transform(
                {
                    "capture_full_height": capture_full_height,
                    "height": height,
                    "image_quality": image_quality,
                    "s3_target_address": s3_target_address,
                    "scroll_all_content": scroll_all_content,
                    "url": url,
                    "wait": wait,
                    "width": width,
                },
                tool_screenshot_webpage_params.ToolScreenshotWebpageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"session_id": session_id}, tool_screenshot_webpage_params.ToolScreenshotWebpageParams
                ),
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncToolsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncToolsResourceWithStreamingResponse(self)

    async def fetch_webpage(
        self,
        *,
        session_id: str | Omit = omit,
        format: Literal["html", "markdown"] | Omit = omit,
        new_page: bool | Omit = omit,
        page_index: int | Omit = omit,
        return_partial_on_timeout: bool | Omit = omit,
        url: str | Omit = omit,
        wait: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Retrieve the rendered content of a webpage, optionally formatted as Markdown or
        HTML.

        Args:
          session_id: An optional browser session identifier to reference an existing running browser
              session. If provided, the tool will execute within that browser session.

          format: The output format of the content.

          new_page: Whether to create a new page for the content.

          page_index: The index of the page to fetch content from. **Overides new_page**.

          return_partial_on_timeout: Whether to return partial content if the content is not loaded within the 20
              seconds.

          url: The URL of the webpage to fetch content from. When left empty, the current
              webpage is used.

          wait: The time to wait for **dynamic** content to load in **milliseconds**.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return await self._post(
            "/v1/tools/fetch-webpage",
            body=await async_maybe_transform(
                {
                    "format": format,
                    "new_page": new_page,
                    "page_index": page_index,
                    "return_partial_on_timeout": return_partial_on_timeout,
                    "url": url,
                    "wait": wait,
                },
                tool_fetch_webpage_params.ToolFetchWebpageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"session_id": session_id}, tool_fetch_webpage_params.ToolFetchWebpageParams
                ),
            ),
            cast_to=str,
        )

    async def perform_web_task(
        self,
        *,
        prompt: str,
        session_id: str | Omit = omit,
        agent: Literal["browser-use", "openai-cua"] | Omit = omit,
        detect_elements: bool | Omit = omit,
        highlight_elements: bool | Omit = omit,
        human_intervention: bool | Omit = omit,
        max_steps: int | Omit = omit,
        model: str | Omit = omit,
        output_schema: object | Omit = omit,
        provider: Literal["openai", "gemini", "groq", "azure", "xai"] | Omit = omit,
        secret_values: Dict[str, str] | Omit = omit,
        url: str | Omit = omit,
        directly_open_url: bool | Omit = omit,
        async_: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolPerformWebTaskResponse:
        """
        Start from a URL and perform the given task.

        Args:
          prompt: The task to be autonomously completed.

          session_id: An optional browser session identifier to reference an existing running browser
              sessions. When passed, the tool will be executed on the provided browser
              session.

          agent: The AI agent to use for task completion. Defaults to browser-use.

          detect_elements: Enable element detection for better interaction accuracy. Improves the agent's
              ability to identify and interact with UI elements.

          highlight_elements: Whether to highlight elements during task execution for better visibility.

          human_intervention: Allow human intervention during task execution. When enabled, the agent can
              request human input for ambiguous situations.

          max_steps: Maximum number of steps the agent can take to complete the task. Defaults
              to 200.

          model: The specific model to use for task completion. see our
              [models](/agentic-browser-control/ai-task-completion#available-models) page for
              more information.

          output_schema: JSON Schema defining the expected structure of the output data.

          provider: The AI provider to use for task completion.

          secret_values: Secret values to pass to the agent for secure credential handling. Keys and
              values are passed as environment variables to the agent.

          url: The URL of the webpage. If not provided, the tool will use the current page in
              the session.

          directly_open_url: If true, the tool will directly open the URL in the browser.

          async_: Whether to execute the task asynchronously.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/tools/perform-web-task",
            body=await async_maybe_transform(
                {
                    "prompt": prompt,
                    "agent": agent,
                    "detect_elements": detect_elements,
                    "highlight_elements": highlight_elements,
                    "human_intervention": human_intervention,
                    "max_steps": max_steps,
                    "model": model,
                    "output_schema": output_schema,
                    "provider": provider,
                    "secret_values": secret_values,
                    "url": url,
                    "directly_open_url": directly_open_url,
                    "async": async_,
                },
                tool_perform_web_task_params.ToolPerformWebTaskParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"session_id": session_id}, tool_perform_web_task_params.ToolPerformWebTaskParams
                ),
            ),
            cast_to=ToolPerformWebTaskResponse,
        )

    async def screenshot_webpage(
        self,
        *,
        session_id: str | Omit = omit,
        capture_full_height: bool | Omit = omit,
        height: int | Omit = omit,
        image_quality: int | Omit = omit,
        s3_target_address: str | Omit = omit,
        scroll_all_content: bool | Omit = omit,
        url: str | Omit = omit,
        wait: int | Omit = omit,
        width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        This endpoint captures a screenshot of the specified webpage using Chromium.
        Users can customize the viewport dimensions and capture options.

        Args:
          session_id: An optional browser session identifier to reference an existing running browser
              sessions. When passed, the tool will be executed on the provided browser
              session.

          capture_full_height: If true, captures the entire height of the page, ignoring the viewport height.

          height: The height of the browser viewport in pixels.

          image_quality: Quality of the output image, on the range 1-100. 100 will not perform any
              compression.

          s3_target_address: Presigned S3 url target to upload the image to.

          scroll_all_content: If true, scrolls the page and captures all visible content.

          url: The URL of the webpage to capture.

          wait: Duration in milliseconds to wait after page has loaded, mainly used for sites
              with JS animations.

          width: The width of the browser viewport in pixels.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "image/png", **(extra_headers or {})}
        return await self._post(
            "/v1/tools/screenshot",
            body=await async_maybe_transform(
                {
                    "capture_full_height": capture_full_height,
                    "height": height,
                    "image_quality": image_quality,
                    "s3_target_address": s3_target_address,
                    "scroll_all_content": scroll_all_content,
                    "url": url,
                    "wait": wait,
                    "width": width,
                },
                tool_screenshot_webpage_params.ToolScreenshotWebpageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"session_id": session_id}, tool_screenshot_webpage_params.ToolScreenshotWebpageParams
                ),
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class ToolsResourceWithRawResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.fetch_webpage = to_raw_response_wrapper(
            tools.fetch_webpage,
        )
        self.perform_web_task = to_raw_response_wrapper(
            tools.perform_web_task,
        )
        self.screenshot_webpage = to_custom_raw_response_wrapper(
            tools.screenshot_webpage,
            BinaryAPIResponse,
        )


class AsyncToolsResourceWithRawResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.fetch_webpage = async_to_raw_response_wrapper(
            tools.fetch_webpage,
        )
        self.perform_web_task = async_to_raw_response_wrapper(
            tools.perform_web_task,
        )
        self.screenshot_webpage = async_to_custom_raw_response_wrapper(
            tools.screenshot_webpage,
            AsyncBinaryAPIResponse,
        )


class ToolsResourceWithStreamingResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.fetch_webpage = to_streamed_response_wrapper(
            tools.fetch_webpage,
        )
        self.perform_web_task = to_streamed_response_wrapper(
            tools.perform_web_task,
        )
        self.screenshot_webpage = to_custom_streamed_response_wrapper(
            tools.screenshot_webpage,
            StreamedBinaryAPIResponse,
        )


class AsyncToolsResourceWithStreamingResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.fetch_webpage = async_to_streamed_response_wrapper(
            tools.fetch_webpage,
        )
        self.perform_web_task = async_to_streamed_response_wrapper(
            tools.perform_web_task,
        )
        self.screenshot_webpage = async_to_custom_streamed_response_wrapper(
            tools.screenshot_webpage,
            AsyncStreamedBinaryAPIResponse,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ApplicationCreateResponse"]


class ApplicationCreateResponse(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the application"""

    created_at: Optional[datetime] = None
    """Timestamp when the application was created"""

    description: Optional[str] = None
    """Description of the application"""

    name: Optional[str] = None
    """Name of the application"""

    url: Optional[str] = None
    """URL of the application"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.extension_list_response import ExtensionListResponse

__all__ = ["ExtensionsResource", "AsyncExtensionsResource"]


class ExtensionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExtensionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return ExtensionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExtensionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return ExtensionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExtensionListResponse:
        """Get all extensions for the authenticated user"""
        return self._get(
            "/v1/extensions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExtensionListResponse,
        )


class AsyncExtensionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExtensionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncExtensionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExtensionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncExtensionsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExtensionListResponse:
        """Get all extensions for the authenticated user"""
        return await self._get(
            "/v1/extensions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExtensionListResponse,
        )


class ExtensionsResourceWithRawResponse:
    def __init__(self, extensions: ExtensionsResource) -> None:
        self._extensions = extensions

        self.list = to_raw_response_wrapper(
            extensions.list,
        )


class AsyncExtensionsResourceWithRawResponse:
    def __init__(self, extensions: AsyncExtensionsResource) -> None:
        self._extensions = extensions

        self.list = async_to_raw_response_wrapper(
            extensions.list,
        )


class ExtensionsResourceWithStreamingResponse:
    def __init__(self, extensions: ExtensionsResource) -> None:
        self._extensions = extensions

        self.list = to_streamed_response_wrapper(
            extensions.list,
        )


class AsyncExtensionsResourceWithStreamingResponse:
    def __init__(self, extensions: AsyncExtensionsResource) -> None:
        self._extensions = extensions

        self.list = async_to_streamed_response_wrapper(
            extensions.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from anchorbrowser import Anchorbrowser, AsyncAnchorbrowser
from anchorbrowser._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPrimary:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_get(self, client: Anchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/session_id/recordings/primary/fetch").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        primary = client.sessions.recordings.primary.get(
            "session_id",
        )
        assert primary.is_closed
        assert primary.json() == {"foo": "bar"}
        assert cast(Any, primary.is_closed) is True
        assert isinstance(primary, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_get(self, client: Anchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/session_id/recordings/primary/fetch").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        primary = client.sessions.recordings.primary.with_raw_response.get(
            "session_id",
        )

        assert primary.is_closed is True
        assert primary.http_request.headers.get("X-Stainless-Lang") == "python"
        assert primary.json() == {"foo": "bar"}
        assert isinstance(primary, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_get(self, client: Anchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/session_id/recordings/primary/fetch").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.sessions.recordings.primary.with_streaming_response.get(
            "session_id",
        ) as primary:
            assert not primary.is_closed
            assert primary.http_request.headers.get("X-Stainless-Lang") == "python"

            assert primary.json() == {"foo": "bar"}
            assert cast(Any, primary.is_closed) is True
            assert isinstance(primary, StreamedBinaryAPIResponse)

        assert cast(Any, primary.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_get(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.recordings.primary.with_raw_response.get(
                "",
            )


class TestAsyncPrimary:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_get(self, async_client: AsyncAnchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/session_id/recordings/primary/fetch").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        primary = await async_client.sessions.recordings.primary.get(
            "session_id",
        )
        assert primary.is_closed
        assert await primary.json() == {"foo": "bar"}
        assert cast(Any, primary.is_closed) is True
        assert isinstance(primary, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_get(self, async_client: AsyncAnchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/session_id/recordings/primary/fetch").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        primary = await async_client.sessions.recordings.primary.with_raw_response.get(
            "session_id",
        )

        assert primary.is_closed is True
        assert primary.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await primary.json() == {"foo": "bar"}
        assert isinstance(primary, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_get(self, async_client: AsyncAnchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/session_id/recordings/primary/fetch").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.sessions.recordings.primary.with_streaming_response.get(
            "session_id",
        ) as primary:
            assert not primary.is_closed
            assert primary.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await primary.json() == {"foo": "bar"}
            assert cast(Any, primary.is_closed) is True
            assert isinstance(primary, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, primary.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_get(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.recordings.primary.with_raw_response.get(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "IdentityCreateParams",
    "Credential",
    "CredentialUsernamePasswordCredentialSchema",
    "CredentialAuthenticatorCredentialSchema",
    "CredentialCustomCredentialSchema",
    "CredentialCustomCredentialSchemaField",
]


class IdentityCreateParams(TypedDict, total=False):
    credentials: Required[Iterable[Credential]]
    """Array of credentials for authentication"""

    source: Required[str]
    """The source URL for the identity (e.g., login page URL)"""

    validate_async: Annotated[bool, PropertyInfo(alias="validateAsync")]
    """Whether to validate the identity asynchronously. Defaults to true."""

    application_description: Annotated[str, PropertyInfo(alias="applicationDescription")]
    """Optional application description"""

    application_name: Annotated[str, PropertyInfo(alias="applicationName")]
    """Optional application name to associate with the identity"""

    metadata: Dict[str, object]
    """Optional metadata for the identity"""

    name: str
    """Name of the identity. Defaults to "Unnamed Identity" if not provided."""


class CredentialUsernamePasswordCredentialSchema(TypedDict, total=False):
    password: str
    """The password of the credential"""

    type: Literal["username_password"]
    """The type of credential"""

    username: str
    """The username of the credential"""


class CredentialAuthenticatorCredentialSchema(TypedDict, total=False):
    otp: str
    """The OTP of the credential"""

    secret: str
    """The secret of the credential"""

    type: Literal["authenticator"]
    """The type of credential"""


class CredentialCustomCredentialSchemaField(TypedDict, total=False):
    name: str
    """The name of the field"""

    value: str
    """The value of the field"""


class CredentialCustomCredentialSchema(TypedDict, total=False):
    fields: Iterable[CredentialCustomCredentialSchemaField]

    type: Literal["custom"]
    """The type of credential"""


Credential: TypeAlias = Union[
    CredentialUsernamePasswordCredentialSchema,
    CredentialAuthenticatorCredentialSchema,
    CredentialCustomCredentialSchema,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ToolScreenshotWebpageParams"]


class ToolScreenshotWebpageParams(TypedDict, total=False):
    session_id: Annotated[str, PropertyInfo(alias="sessionId")]
    """
    An optional browser session identifier to reference an existing running browser
    sessions. When passed, the tool will be executed on the provided browser
    session.
    """

    capture_full_height: bool
    """If true, captures the entire height of the page, ignoring the viewport height."""

    height: int
    """The height of the browser viewport in pixels."""

    image_quality: int
    """Quality of the output image, on the range 1-100.

    100 will not perform any compression.
    """

    s3_target_address: str
    """Presigned S3 url target to upload the image to."""

    scroll_all_content: bool
    """If true, scrolls the page and captures all visible content."""

    url: str
    """The URL of the webpage to capture."""

    wait: int
    """
    Duration in milliseconds to wait after page has loaded, mainly used for sites
    with JS animations.
    """

    width: int
    """The width of the browser viewport in pixels."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ApplicationCreateIdentityTokenResponse", "Data"]


class Data(BaseModel):
    token: Optional[str] = None
    """The generated identity token for authentication"""

    expires_at: Optional[datetime] = None
    """The timestamp when the token expires"""

    token_hash: Optional[str] = None
    """A hash of the token for verification purposes"""


class ApplicationCreateIdentityTokenResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["AuthFlowUpdateResponse", "CustomField"]


class CustomField(BaseModel):
    name: str
    """Name of the custom field"""


class AuthFlowUpdateResponse(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the authentication flow"""

    created_at: Optional[datetime] = None
    """Timestamp when the authentication flow was created"""

    custom_fields: Optional[List[CustomField]] = None
    """Custom fields for this authentication flow"""

    description: Optional[str] = None
    """Description of the authentication flow"""

    is_recommended: Optional[bool] = None
    """Whether this is the recommended authentication flow"""

    methods: Optional[List[str]] = None
    """Authentication methods in this flow"""

    name: Optional[str] = None
    """Name of the authentication flow"""

    updated_at: Optional[datetime] = None
    """Timestamp when the authentication flow was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from tests.utils import assert_matches_type
from anchorbrowser import Anchorbrowser, AsyncAnchorbrowser
from anchorbrowser.types import (
    SessionGotoResponse,
    SessionCreateResponse,
    SessionScrollResponse,
    SessionRetrieveResponse,
    SessionUploadFileResponse,
    SessionDragAndDropResponse,
    SessionRetrieveDownloadsResponse,
)
from anchorbrowser._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)
from anchorbrowser.types.shared import SuccessResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSessions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Anchorbrowser) -> None:
        session = client.sessions.create()
        assert_matches_type(SessionCreateResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Anchorbrowser) -> None:
        session = client.sessions.create(
            browser={
                "adblock": {"active": True},
                "captcha_solver": {"active": True},
                "disable_web_security": {"active": True},
                "extensions": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                "extra_stealth": {"active": True},
                "fullscreen": {"active": True},
                "headless": {"active": True},
                "p2p_download": {"active": True},
                "popup_blocker": {"active": True},
                "profile": {
                    "name": "name",
                    "persist": True,
                },
                "viewport": {
                    "height": 0,
                    "width": 0,
                },
            },
            identities=[{"id": "123e4567-e89b-12d3-a456-426614174000"}],
            integrations=[
                {
                    "id": "550e8400-e29b-41d4-a716-446655440000",
                    "configuration": {"load_mode": "all"},
                    "type": "1PASSWORD",
                }
            ],
            session={
                "initial_url": "https://example.com",
                "live_view": {"read_only": True},
                "proxy": {
                    "active": True,
                    "city": "city",
                    "country_code": "af",
                    "region": "region",
                    "type": "anchor_proxy",
                },
                "recording": {"active": True},
                "timeout": {
                    "idle_timeout": 0,
                    "max_duration": 0,
                },
            },
        )
        assert_matches_type(SessionCreateResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Anchorbrowser) -> None:
        response = client.sessions.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionCreateResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Anchorbrowser) -> None:
        with client.sessions.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionCreateResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Anchorbrowser) -> None:
        session = client.sessions.retrieve(
            "session_id",
        )
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Anchorbrowser) -> None:
        response = client.sessions.with_raw_response.retrieve(
            "session_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Anchorbrowser) -> None:
        with client.sessions.with_streaming_response.retrieve(
            "session_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionRetrieveResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Anchorbrowser) -> None:
        session = client.sessions.delete(
            "session_id",
        )
        assert_matches_type(SuccessResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Anchorbrowser) -> None:
        response = client.sessions.with_raw_response.delete(
            "session_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SuccessResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Anchorbrowser) -> None:
        with client.sessions.with_streaming_response.delete(
            "session_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SuccessResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_drag_and_drop(self, client: Anchorbrowser) -> None:
        session = client.sessions.drag_and_drop(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        )
        assert_matches_type(SessionDragAndDropResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_drag_and_drop_with_all_params(self, client: Anchorbrowser) -> None:
        session = client.sessions.drag_and_drop(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
            button="left",
        )
        assert_matches_type(SessionDragAndDropResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_drag_and_drop(self, client: Anchorbrowser) -> None:
        response = client.sessions.with_raw_response.drag_and_drop(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionDragAndDropResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_drag_and_drop(self, client: Anchorbrowser) -> None:
        with client.sessions.with_streaming_response.drag_and_drop(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionDragAndDropResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_drag_and_drop(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.with_raw_response.drag_and_drop(
                session_id="",
                end_x=0,
                end_y=0,
                start_x=0,
                start_y=0,
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_goto(self, client: Anchorbrowser) -> None:
        session = client.sessions.goto(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            url="url",
        )
        assert_matches_type(SessionGotoResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_goto(self, client: Anchorbrowser) -> None:
        response = client.sessions.with_raw_response.goto(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionGotoResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_goto(self, client: Anchorbrowser) -> None:
        with client.sessions.with_streaming_response.goto(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionGotoResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_goto(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.with_raw_response.goto(
                session_id="",
                url="url",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_downloads(self, client: Anchorbrowser) -> None:
        session = client.sessions.retrieve_downloads(
            "session_id",
        )
        assert_matches_type(SessionRetrieveDownloadsResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_downloads(self, client: Anchorbrowser) -> None:
        response = client.sessions.with_raw_response.retrieve_downloads(
            "session_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionRetrieveDownloadsResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_downloads(self, client: Anchorbrowser) -> None:
        with client.sessions.with_streaming_response.retrieve_downloads(
            "session_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionRetrieveDownloadsResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_downloads(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.with_raw_response.retrieve_downloads(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_screenshot(self, client: Anchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/screenshot").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        session = client.sessions.retrieve_screenshot(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert session.is_closed
        assert session.json() == {"foo": "bar"}
        assert cast(Any, session.is_closed) is True
        assert isinstance(session, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_retrieve_screenshot(self, client: Anchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/screenshot").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        session = client.sessions.with_raw_response.retrieve_screenshot(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert session.is_closed is True
        assert session.http_request.headers.get("X-Stainless-Lang") == "python"
        assert session.json() == {"foo": "bar"}
        assert isinstance(session, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_retrieve_screenshot(self, client: Anchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/screenshot").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.sessions.with_streaming_response.retrieve_screenshot(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as session:
            assert not session.is_closed
            assert session.http_request.headers.get("X-Stainless-Lang") == "python"

            assert session.json() == {"foo": "bar"}
            assert cast(Any, session.is_closed) is True
            assert isinstance(session, StreamedBinaryAPIResponse)

        assert cast(Any, session.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_retrieve_screenshot(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.with_raw_response.retrieve_screenshot(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_scroll(self, client: Anchorbrowser) -> None:
        session = client.sessions.scroll(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            delta_y=0,
            x=0,
            y=0,
        )
        assert_matches_type(SessionScrollResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_scroll_with_all_params(self, client: Anchorbrowser) -> None:
        session = client.sessions.scroll(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            delta_y=0,
            x=0,
            y=0,
            delta_x=0,
            steps=0,
            use_os=True,
        )
        assert_matches_type(SessionScrollResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_scroll(self, client: Anchorbrowser) -> None:
        response = client.sessions.with_raw_response.scroll(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            delta_y=0,
            x=0,
            y=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionScrollResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_scroll(self, client: Anchorbrowser) -> None:
        with client.sessions.with_streaming_response.scroll(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            delta_y=0,
            x=0,
            y=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionScrollResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_scroll(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.with_raw_response.scroll(
                session_id="",
                delta_y=0,
                x=0,
                y=0,
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload_file(self, client: Anchorbrowser) -> None:
        session = client.sessions.upload_file(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            file=b"raw file contents",
        )
        assert_matches_type(SessionUploadFileResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_upload_file(self, client: Anchorbrowser) -> None:
        response = client.sessions.with_raw_response.upload_file(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = response.parse()
        assert_matches_type(SessionUploadFileResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_upload_file(self, client: Anchorbrowser) -> None:
        with client.sessions.with_streaming_response.upload_file(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = response.parse()
            assert_matches_type(SessionUploadFileResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_upload_file(self, client: Anchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            client.sessions.with_raw_response.upload_file(
                session_id="",
                file=b"raw file contents",
            )


class TestAsyncSessions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.create()
        assert_matches_type(SessionCreateResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.create(
            browser={
                "adblock": {"active": True},
                "captcha_solver": {"active": True},
                "disable_web_security": {"active": True},
                "extensions": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                "extra_stealth": {"active": True},
                "fullscreen": {"active": True},
                "headless": {"active": True},
                "p2p_download": {"active": True},
                "popup_blocker": {"active": True},
                "profile": {
                    "name": "name",
                    "persist": True,
                },
                "viewport": {
                    "height": 0,
                    "width": 0,
                },
            },
            identities=[{"id": "123e4567-e89b-12d3-a456-426614174000"}],
            integrations=[
                {
                    "id": "550e8400-e29b-41d4-a716-446655440000",
                    "configuration": {"load_mode": "all"},
                    "type": "1PASSWORD",
                }
            ],
            session={
                "initial_url": "https://example.com",
                "live_view": {"read_only": True},
                "proxy": {
                    "active": True,
                    "city": "city",
                    "country_code": "af",
                    "region": "region",
                    "type": "anchor_proxy",
                },
                "recording": {"active": True},
                "timeout": {
                    "idle_timeout": 0,
                    "max_duration": 0,
                },
            },
        )
        assert_matches_type(SessionCreateResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.sessions.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionCreateResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.sessions.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionCreateResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.retrieve(
            "session_id",
        )
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.sessions.with_raw_response.retrieve(
            "session_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionRetrieveResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.sessions.with_streaming_response.retrieve(
            "session_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionRetrieveResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.delete(
            "session_id",
        )
        assert_matches_type(SuccessResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.sessions.with_raw_response.delete(
            "session_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SuccessResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.sessions.with_streaming_response.delete(
            "session_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SuccessResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_drag_and_drop(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.drag_and_drop(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        )
        assert_matches_type(SessionDragAndDropResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_drag_and_drop_with_all_params(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.drag_and_drop(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
            button="left",
        )
        assert_matches_type(SessionDragAndDropResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_drag_and_drop(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.sessions.with_raw_response.drag_and_drop(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionDragAndDropResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_drag_and_drop(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.sessions.with_streaming_response.drag_and_drop(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            end_x=0,
            end_y=0,
            start_x=0,
            start_y=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionDragAndDropResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_drag_and_drop(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.with_raw_response.drag_and_drop(
                session_id="",
                end_x=0,
                end_y=0,
                start_x=0,
                start_y=0,
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_goto(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.goto(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            url="url",
        )
        assert_matches_type(SessionGotoResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_goto(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.sessions.with_raw_response.goto(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionGotoResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_goto(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.sessions.with_streaming_response.goto(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionGotoResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_goto(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.with_raw_response.goto(
                session_id="",
                url="url",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_downloads(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.retrieve_downloads(
            "session_id",
        )
        assert_matches_type(SessionRetrieveDownloadsResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_downloads(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.sessions.with_raw_response.retrieve_downloads(
            "session_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionRetrieveDownloadsResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_downloads(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.sessions.with_streaming_response.retrieve_downloads(
            "session_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionRetrieveDownloadsResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_downloads(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.with_raw_response.retrieve_downloads(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_screenshot(self, async_client: AsyncAnchorbrowser, respx_mock: MockRouter) -> None:
        respx_mock.get("/v1/sessions/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/screenshot").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        session = await async_client.sessions.retrieve_screenshot(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert session.is_closed
        assert await session.json() == {"foo": "bar"}
        assert cast(Any, session.is_closed) is True
        assert isinstance(session, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_retrieve_screenshot(
        self, async_client: AsyncAnchorbrowser, respx_mock: MockRouter
    ) -> None:
        respx_mock.get("/v1/sessions/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/screenshot").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        session = await async_client.sessions.with_raw_response.retrieve_screenshot(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert session.is_closed is True
        assert session.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await session.json() == {"foo": "bar"}
        assert isinstance(session, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_retrieve_screenshot(
        self, async_client: AsyncAnchorbrowser, respx_mock: MockRouter
    ) -> None:
        respx_mock.get("/v1/sessions/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/screenshot").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.sessions.with_streaming_response.retrieve_screenshot(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as session:
            assert not session.is_closed
            assert session.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await session.json() == {"foo": "bar"}
            assert cast(Any, session.is_closed) is True
            assert isinstance(session, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, session.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_retrieve_screenshot(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.with_raw_response.retrieve_screenshot(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_scroll(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.scroll(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            delta_y=0,
            x=0,
            y=0,
        )
        assert_matches_type(SessionScrollResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_scroll_with_all_params(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.scroll(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            delta_y=0,
            x=0,
            y=0,
            delta_x=0,
            steps=0,
            use_os=True,
        )
        assert_matches_type(SessionScrollResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_scroll(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.sessions.with_raw_response.scroll(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            delta_y=0,
            x=0,
            y=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionScrollResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_scroll(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.sessions.with_streaming_response.scroll(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            delta_y=0,
            x=0,
            y=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionScrollResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_scroll(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.with_raw_response.scroll(
                session_id="",
                delta_y=0,
                x=0,
                y=0,
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload_file(self, async_client: AsyncAnchorbrowser) -> None:
        session = await async_client.sessions.upload_file(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            file=b"raw file contents",
        )
        assert_matches_type(SessionUploadFileResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_upload_file(self, async_client: AsyncAnchorbrowser) -> None:
        response = await async_client.sessions.with_raw_response.upload_file(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        session = await response.parse()
        assert_matches_type(SessionUploadFileResponse, session, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_upload_file(self, async_client: AsyncAnchorbrowser) -> None:
        async with async_client.sessions.with_streaming_response.upload_file(
            session_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            session = await response.parse()
            assert_matches_type(SessionUploadFileResponse, session, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_upload_file(self, async_client: AsyncAnchorbrowser) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `session_id` but received ''"):
            await async_client.sessions.with_raw_response.upload_file(
                session_id="",
                file=b"raw file contents",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import (
    application_list_params,
    application_create_params,
    application_list_identities_params,
    application_create_identity_token_params,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .auth_flows import (
    AuthFlowsResource,
    AsyncAuthFlowsResource,
    AuthFlowsResourceWithRawResponse,
    AsyncAuthFlowsResourceWithRawResponse,
    AuthFlowsResourceWithStreamingResponse,
    AsyncAuthFlowsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.application_list_response import ApplicationListResponse
from ...types.application_create_response import ApplicationCreateResponse
from ...types.application_delete_response import ApplicationDeleteResponse
from ...types.application_retrieve_response import ApplicationRetrieveResponse
from ...types.application_list_identities_response import ApplicationListIdentitiesResponse
from ...types.application_create_identity_token_response import ApplicationCreateIdentityTokenResponse

__all__ = ["ApplicationsResource", "AsyncApplicationsResource"]


class ApplicationsResource(SyncAPIResource):
    @cached_property
    def auth_flows(self) -> AuthFlowsResource:
        return AuthFlowsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ApplicationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return ApplicationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return ApplicationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        source: str,
        description: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationCreateResponse:
        """Creates a new application for identity management.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          source: The source URL of the application

          description: Description of the application

          name: Name of the application

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/applications",
            body=maybe_transform(
                {
                    "source": source,
                    "description": description,
                    "name": name,
                },
                application_create_params.ApplicationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationCreateResponse,
        )

    def retrieve(
        self,
        application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationRetrieveResponse:
        """Retrieves details of a specific application by its ID.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._get(
            f"/v1/applications/{application_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationRetrieveResponse,
        )

    def list(
        self,
        *,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationListResponse:
        """Retrieves all applications for the authenticated team.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          search: Search query to filter applications by name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/applications",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"search": search}, application_list_params.ApplicationListParams),
            ),
            cast_to=ApplicationListResponse,
        )

    def delete(
        self,
        application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationDeleteResponse:
        """Deletes an existing application.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._delete(
            f"/v1/applications/{application_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationDeleteResponse,
        )

    def create_identity_token(
        self,
        application_id: str,
        *,
        callback_url: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationCreateIdentityTokenResponse:
        """Creates an identity token for a specific application.

        This token is used to
        initiate an authentication flow for linking user identities to the application.

        The callback URL must use HTTPS and is where the user will be redirected after
        authentication.

        Args:
          callback_url: The HTTPS URL where the user will be redirected after authentication. Must use
              HTTPS protocol.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._post(
            f"/v1/applications/{application_id}/tokens",
            body=maybe_transform(
                {"callback_url": callback_url},
                application_create_identity_token_params.ApplicationCreateIdentityTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationCreateIdentityTokenResponse,
        )

    def list_identities(
        self,
        application_id: str,
        *,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationListIdentitiesResponse:
        """
        Retrieves all identities associated with a specific application.

        **Beta** Capability. [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          search: Search query to filter identities by name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._get(
            f"/v1/applications/{application_id}/identities",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"search": search}, application_list_identities_params.ApplicationListIdentitiesParams
                ),
            ),
            cast_to=ApplicationListIdentitiesResponse,
        )


class AsyncApplicationsResource(AsyncAPIResource):
    @cached_property
    def auth_flows(self) -> AsyncAuthFlowsResource:
        return AsyncAuthFlowsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncApplicationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncApplicationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        source: str,
        description: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationCreateResponse:
        """Creates a new application for identity management.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          source: The source URL of the application

          description: Description of the application

          name: Name of the application

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/applications",
            body=await async_maybe_transform(
                {
                    "source": source,
                    "description": description,
                    "name": name,
                },
                application_create_params.ApplicationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationCreateResponse,
        )

    async def retrieve(
        self,
        application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationRetrieveResponse:
        """Retrieves details of a specific application by its ID.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._get(
            f"/v1/applications/{application_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationRetrieveResponse,
        )

    async def list(
        self,
        *,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationListResponse:
        """Retrieves all applications for the authenticated team.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          search: Search query to filter applications by name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/applications",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"search": search}, application_list_params.ApplicationListParams),
            ),
            cast_to=ApplicationListResponse,
        )

    async def delete(
        self,
        application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationDeleteResponse:
        """Deletes an existing application.

        **Beta** Capability.

        [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._delete(
            f"/v1/applications/{application_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationDeleteResponse,
        )

    async def create_identity_token(
        self,
        application_id: str,
        *,
        callback_url: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationCreateIdentityTokenResponse:
        """Creates an identity token for a specific application.

        This token is used to
        initiate an authentication flow for linking user identities to the application.

        The callback URL must use HTTPS and is where the user will be redirected after
        authentication.

        Args:
          callback_url: The HTTPS URL where the user will be redirected after authentication. Must use
              HTTPS protocol.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._post(
            f"/v1/applications/{application_id}/tokens",
            body=await async_maybe_transform(
                {"callback_url": callback_url},
                application_create_identity_token_params.ApplicationCreateIdentityTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationCreateIdentityTokenResponse,
        )

    async def list_identities(
        self,
        application_id: str,
        *,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationListIdentitiesResponse:
        """
        Retrieves all identities associated with a specific application.

        **Beta** Capability. [Contact support](mailto:support@anchorbrowser.io) to
        enable.

        Args:
          search: Search query to filter identities by name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._get(
            f"/v1/applications/{application_id}/identities",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"search": search}, application_list_identities_params.ApplicationListIdentitiesParams
                ),
            ),
            cast_to=ApplicationListIdentitiesResponse,
        )


class ApplicationsResourceWithRawResponse:
    def __init__(self, applications: ApplicationsResource) -> None:
        self._applications = applications

        self.create = to_raw_response_wrapper(
            applications.create,
        )
        self.retrieve = to_raw_response_wrapper(
            applications.retrieve,
        )
        self.list = to_raw_response_wrapper(
            applications.list,
        )
        self.delete = to_raw_response_wrapper(
            applications.delete,
        )
        self.create_identity_token = to_raw_response_wrapper(
            applications.create_identity_token,
        )
        self.list_identities = to_raw_response_wrapper(
            applications.list_identities,
        )

    @cached_property
    def auth_flows(self) -> AuthFlowsResourceWithRawResponse:
        return AuthFlowsResourceWithRawResponse(self._applications.auth_flows)


class AsyncApplicationsResourceWithRawResponse:
    def __init__(self, applications: AsyncApplicationsResource) -> None:
        self._applications = applications

        self.create = async_to_raw_response_wrapper(
            applications.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            applications.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            applications.list,
        )
        self.delete = async_to_raw_response_wrapper(
            applications.delete,
        )
        self.create_identity_token = async_to_raw_response_wrapper(
            applications.create_identity_token,
        )
        self.list_identities = async_to_raw_response_wrapper(
            applications.list_identities,
        )

    @cached_property
    def auth_flows(self) -> AsyncAuthFlowsResourceWithRawResponse:
        return AsyncAuthFlowsResourceWithRawResponse(self._applications.auth_flows)


class ApplicationsResourceWithStreamingResponse:
    def __init__(self, applications: ApplicationsResource) -> None:
        self._applications = applications

        self.create = to_streamed_response_wrapper(
            applications.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            applications.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            applications.list,
        )
        self.delete = to_streamed_response_wrapper(
            applications.delete,
        )
        self.create_identity_token = to_streamed_response_wrapper(
            applications.create_identity_token,
        )
        self.list_identities = to_streamed_response_wrapper(
            applications.list_identities,
        )

    @cached_property
    def auth_flows(self) -> AuthFlowsResourceWithStreamingResponse:
        return AuthFlowsResourceWithStreamingResponse(self._applications.auth_flows)


class AsyncApplicationsResourceWithStreamingResponse:
    def __init__(self, applications: AsyncApplicationsResource) -> None:
        self._applications = applications

        self.create = async_to_streamed_response_wrapper(
            applications.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            applications.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            applications.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            applications.delete,
        )
        self.create_identity_token = async_to_streamed_response_wrapper(
            applications.create_identity_token,
        )
        self.list_identities = async_to_streamed_response_wrapper(
            applications.list_identities,
        )

    @cached_property
    def auth_flows(self) -> AsyncAuthFlowsResourceWithStreamingResponse:
        return AsyncAuthFlowsResourceWithStreamingResponse(self._applications.auth_flows)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["AllStatusResponse", "Data", "DataItem"]


class DataItem(BaseModel):
    created_at: datetime
    """Timestamp when the browser session was created"""

    session_id: str
    """Unique identifier for the browser session"""

    status: str
    """Current status of the browser session"""


class Data(BaseModel):
    count: Optional[int] = None
    """Total number of browser sessions"""

    items: Optional[List[DataItem]] = None


class AllStatusResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .auth_flows import (
    AuthFlowsResource,
    AsyncAuthFlowsResource,
    AuthFlowsResourceWithRawResponse,
    AsyncAuthFlowsResourceWithRawResponse,
    AuthFlowsResourceWithStreamingResponse,
    AsyncAuthFlowsResourceWithStreamingResponse,
)
from .applications import (
    ApplicationsResource,
    AsyncApplicationsResource,
    ApplicationsResourceWithRawResponse,
    AsyncApplicationsResourceWithRawResponse,
    ApplicationsResourceWithStreamingResponse,
    AsyncApplicationsResourceWithStreamingResponse,
)

__all__ = [
    "AuthFlowsResource",
    "AsyncAuthFlowsResource",
    "AuthFlowsResourceWithRawResponse",
    "AsyncAuthFlowsResourceWithRawResponse",
    "AuthFlowsResourceWithStreamingResponse",
    "AsyncAuthFlowsResourceWithStreamingResponse",
    "ApplicationsResource",
    "AsyncApplicationsResource",
    "ApplicationsResourceWithRawResponse",
    "AsyncApplicationsResourceWithRawResponse",
    "ApplicationsResourceWithStreamingResponse",
    "AsyncApplicationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ApplicationRetrieveResponse"]


class ApplicationRetrieveResponse(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the application"""

    allowed_domains: Optional[List[str]] = None
    """List of allowed domains for this application"""

    auth_methods: Optional[List[str]] = None
    """Authentication methods available for this application"""

    created_at: Optional[datetime] = None
    """Timestamp when the application was created"""

    description: Optional[str] = None
    """Description of the application"""

    identity_count: Optional[int] = None
    """Number of identities associated with this application"""

    name: Optional[str] = None
    """Name of the application"""

    updated_at: Optional[datetime] = None
    """Timestamp when the application was last updated"""

    url: Optional[str] = None
    """URL of the application"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TaskRunResponse", "Data"]


class Data(BaseModel):
    message: str
    """Execution result message"""

    success: bool
    """Whether the task executed successfully"""

    task_id: str = FieldInfo(alias="taskId")
    """Task identifier"""

    error: Optional[str] = None
    """Error message if execution failed"""

    execution_time: Optional[float] = FieldInfo(alias="executionTime", default=None)
    """Execution duration in milliseconds"""

    output: Optional[str] = None
    """Task execution output"""


class TaskRunResponse(BaseModel):
    data: Optional[Data] = None




# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["AuthFlowCreateParams", "CustomField"]


class AuthFlowCreateParams(TypedDict, total=False):
    methods: Required[List[Literal["username_password", "authenticator", "custom"]]]
    """Authentication methods in this flow"""

    name: Required[str]
    """Name of the authentication flow"""

    custom_fields: Iterable[CustomField]
    """Custom fields for this authentication flow"""

    description: str
    """Description of the authentication flow"""

    is_recommended: bool
    """Whether this is the recommended authentication flow"""


class CustomField(TypedDict, total=False):
    name: Required[str]
    """Name of the custom field"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .primary import (
    PrimaryResource,
    AsyncPrimaryResource,
    PrimaryResourceWithRawResponse,
    AsyncPrimaryResourceWithRawResponse,
    PrimaryResourceWithStreamingResponse,
    AsyncPrimaryResourceWithStreamingResponse,
)
from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.sessions.recording_list_response import RecordingListResponse

__all__ = ["RecordingsResource", "AsyncRecordingsResource"]


class RecordingsResource(SyncAPIResource):
    @cached_property
    def primary(self) -> PrimaryResource:
        return PrimaryResource(self._client)

    @cached_property
    def with_raw_response(self) -> RecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return RecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return RecordingsResourceWithStreamingResponse(self)

    def list(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingListResponse:
        """Retrieves the URLs of the browser session's video recordings.

        Requires a valid
        API key for authentication.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            f"/v1/sessions/{session_id}/recordings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingListResponse,
        )


class AsyncRecordingsResource(AsyncAPIResource):
    @cached_property
    def primary(self) -> AsyncPrimaryResource:
        return AsyncPrimaryResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#accessing-raw-response-data-eg-headers
        """
        return AsyncRecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/anchorbrowser/AnchorBrowser-SDK-Python#with_streaming_response
        """
        return AsyncRecordingsResourceWithStreamingResponse(self)

    async def list(
        self,
        session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingListResponse:
        """Retrieves the URLs of the browser session's video recordings.

        Requires a valid
        API key for authentication.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            f"/v1/sessions/{session_id}/recordings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingListResponse,
        )


class RecordingsResourceWithRawResponse:
    def __init__(self, recordings: RecordingsResource) -> None:
        self._recordings = recordings

        self.list = to_raw_response_wrapper(
            recordings.list,
        )

    @cached_property
    def primary(self) -> PrimaryResourceWithRawResponse:
        return PrimaryResourceWithRawResponse(self._recordings.primary)


class AsyncRecordingsResourceWithRawResponse:
    def __init__(self, recordings: AsyncRecordingsResource) -> None:
        self._recordings = recordings

        self.list = async_to_raw_response_wrapper(
            recordings.list,
        )

    @cached_property
    def primary(self) -> AsyncPrimaryResourceWithRawResponse:
        return AsyncPrimaryResourceWithRawResponse(self._recordings.primary)


class RecordingsResourceWithStreamingResponse:
    def __init__(self, recordings: RecordingsResource) -> None:
        self._recordings = recordings

        self.list = to_streamed_response_wrapper(
            recordings.list,
        )

    @cached_property
    def primary(self) -> PrimaryResourceWithStreamingResponse:
        return PrimaryResourceWithStreamingResponse(self._recordings.primary)


class AsyncRecordingsResourceWithStreamingResponse:
    def __init__(self, recordings: AsyncRecordingsResource) -> None:
        self._recordings = recordings

        self.list = async_to_streamed_response_wrapper(
            recordings.list,
        )

    @cached_property
    def primary(self) -> AsyncPrimaryResourceWithStreamingResponse:
        return AsyncPrimaryResourceWithStreamingResponse(self._recordings.primary)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ApplicationCreateParams"]


class ApplicationCreateParams(TypedDict, total=False):
    source: Required[str]
    """The source URL of the application"""

    description: str
    """Description of the application"""

    name: str
    """Name of the application"""
